/***************************************************************************************/
/*                                                                                     */
/*  Girder 3.0 Plugin                                                                  */
/*  User interface                                                                     */
/*                                                                                     */
/*  Copyright 2000 (c) Ron Bessems                                                     */
/*  GNU General Public License                                                         */
/*                                                                                     */
/*                                                                                     */
/***************************************************************************************/

#define WIN32_LEAN_AND_MEAN
#define GIRDER_CPP

#include <windows.h>
#include "girder.h"
#include "ui.h"
#include <stdio.h>
#include <malloc.h>
#include <time.h>

#include "lcdriver.h"

#ifdef _MSC_VER
  
#define __export

#else

#define __export _export

#endif

t_functions Functions;
p_command CurCommand;
HINSTANCE hInstance;

int nDisplayCols = 0, nDisplayRows = 0;
LPSTR pDisplayBuf = NULL;
BOOL bDisplayOpen = FALSE;

BOOL DisplayOpen(PCHAR retbuf, int bufsize, LPSTR BufferToSet = NULL );


BOOL DisplayOpen(PCHAR retbuf, int bufsize, LPSTR BufferToSet )
{
  if (!bDisplayOpen) {
    if (LCD_RESULT_NOERROR != lcdRequest()) {
      strncpy(retbuf, "Could not open LCD.", bufsize);
      return FALSE; 
    }

    nDisplayCols = lcdGetWidth();
    nDisplayRows = lcdGetHeight();
    pDisplayBuf = (LPSTR)malloc(nDisplayCols * nDisplayRows);
	if( BufferToSet )
	{
		memcpy(pDisplayBuf, BufferToSet, nDisplayCols * nDisplayRows);
	}
	else
	{
		memset(pDisplayBuf, ' ', nDisplayCols * nDisplayRows);
        lcdClearDisplay();            // Clear
	}
    lcdSetWrapMode(0, 0);         // Turn off wrap mode
    lcdSetCursor(0);              // Hide cursor

    bDisplayOpen = TRUE;
  }
  return bDisplayOpen;
}

void DisplayClose()
{
  if (bDisplayOpen) {
    free(pDisplayBuf);
    pDisplayBuf = NULL;

    lcdFree();

    bDisplayOpen = FALSE;
  }
}

void DisplayClose(p_command command, PCHAR retbuf, int bufsize)
{
  DisplayClose();
  strncpy(retbuf, "Display closed.", bufsize);
}

void DisplayClear(p_command command, PCHAR retbuf, int bufsize)
{
  if (!bDisplayOpen && !DisplayOpen(retbuf, bufsize)) 
    return;
  lcdClearDisplay();
  memset(pDisplayBuf, ' ', nDisplayCols * nDisplayRows);
  strncpy(retbuf, "Display cleared.", bufsize);
}

void DisplayCommon(LPCSTR str, p_command command, PCHAR retbuf, int bufsize)
{
  if (!DisplayOpen(retbuf, bufsize)) 
    return;
  strncpy(retbuf, str, bufsize);

  int row = command->lvalue1 % nDisplayRows;
  int col = command->lvalue2 % nDisplayCols;
  int width = nDisplayCols - col;
  if ((command->lvalue3 > 0) && (command->lvalue3 < width))
    width = command->lvalue3;
  LPSTR start = NULL, end;
  LPSTR bp = pDisplayBuf + (row * nDisplayCols) + col;
  
  while (width-- > 0) {
    char ch = *str;
    if ('\0' == ch)
      ch = ' ';
    else
      str++;
    if (ch != *bp) {
      if (NULL == start)
        start = bp;
      end = bp;
      *bp = ch;
    }
    else if (NULL == start)
      col++;
    bp++;
  }
  if (NULL == start) 
    return;    // No change

  end++;
  lcdSetCursorPos(col, row);
  lcdSendString(start, (end - start));
}

// umsetzung Zeichensatz

void ReplaceCharset( char* buf, int buflen )
{
	char* p = buf;
	int i = buflen;

	while( *p && i > 0 )
	{
		switch( *p )
		{
			case '':
			case '':
			  *p = (char)0xE1;
			  break;

			case '':
			case '':
			  *p = (char)0xEF;
			  break;

			case '':
			case '':
			  *p = (char)0xF5;
			  break;

			case '':
			  *p = (char)0xE2;
			  break;

			case '':
			  *p=  '\'';
			  break;

			case '\\':
			  *p = (char)0x2F;  //evtl a4
			  break;

		} // switch
		p++;
		i--;
	} // while
}

void DisplayString(p_command command, PCHAR retbuf, int bufsize)
{
  char buf[256];
  Functions.parse_reg_string((command->svalue1), buf, sizeof(buf));
  ReplaceCharset( buf, sizeof(buf) );
  DisplayCommon(buf, command, retbuf, bufsize);
}

void DisplayStringRegister(p_command command, PCHAR retbuf, int bufsize)
{
  char buf[256];
  sprintf(buf,"[str%d]",command->ivalue2);
  Functions.get_string_var(buf, buf, sizeof(buf));
  ReplaceCharset( buf, sizeof(buf) );
  DisplayCommon(buf, command, retbuf, bufsize);
}


// support for different localities

typedef enum { loc_Default, loc_Germany } LOCALITY;

PCHAR default_month_short[] = { "May", "Oct", "Dec", NULL }; 
PCHAR german_month_short[] = { "Mai", "Okt", "Dez", NULL }; 

PCHAR default_month_long[] = { "January", "February", "March", "May", "June", "July", "October", "December", NULL }; 
PCHAR german_month_long[] = { "Januar", "Februar", "Mrz", "Mai", "Juni", "Juli", "Oktober", "Dezember", NULL }; 

PCHAR default_weekdays_short[] = { "Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat" }; 
PCHAR german_weekdays_short[] = { "So", "Mo", "Di", "Mi", "Do", "Fr", "Sa" }; 

PCHAR default_weekdays_long[] = { "Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday" }; 
PCHAR german_weekdays_long[] = { "Sonntag", "Montag", "Dienstag", "Mittwoch", "Donnerstag", "Freitag", "Samstag" }; 

void replace( char* oldstr, char* newstr, char* buf, int len )
{
  int delta = strlen( newstr ) - strlen( oldstr );
  int actlen = strlen( buf );

  char* p = strstr( buf, oldstr );
  while( p && actlen+delta < len )
  {
	  if( delta > 0 )
	  {
		  memmove( p+delta, p, strlen( p ) + 1 );
	  }
	  if( delta < 0 )
	  {
		  memmove( p, p-delta, strlen( p-delta ) + 1 );
	  }
	  strncpy( p, newstr, strlen( newstr) );
	  p++;
	  actlen += delta;
	  p = strstr( buf, oldstr );
  }
}

void DisplayCurrentTime(p_command command, PCHAR retbuf, int bufsize)
{
  const char *fmt = command->svalue1;
  LOCALITY loc = loc_Default;
  if ((NULL == fmt) || ('\0' == *fmt))
    fmt = "%H:%M:%S";

  if( strncmp( fmt, "%lg", 3 ) == 0 ) // means localize format for germany
  {
	  fmt += 3;
	  loc = loc_Germany;
  }

  time_t ltime;
  struct tm *now;
  int buflen = strlen(fmt)*2 + 128;
  char* buf = (char*)malloc( buflen +1 );
  
  time(&ltime);
  now = localtime(&ltime);
  strftime(buf, buflen, fmt, now);
  if( loc == loc_Germany )
  {
	  int i = 0;
	  if( strstr( fmt, "%A" ) ) // replace long weekdays
	  {
		for( i= 0; i < 7; i++ )
			replace( default_weekdays_long[i], german_weekdays_long[i], buf, buflen );
	  }
	  if( strstr( fmt, "%a" ) ) // replace short weekdays
	  {
		for( i= 0; i < 7; i++ )
			replace( default_weekdays_short[i], german_weekdays_short[i], buf, buflen );
	  }
	  if( strstr( fmt, "%B" ) ) // replace long month
	  {
		for( i= 0; default_month_long[i]; i++ )
			replace( default_month_long[i], german_month_long[i], buf, buflen );
	  }
	  if( strstr( fmt, "%b" ) ) // replace short month
	  {
		for( i= 0; default_month_short[i]; i++ )
			replace( default_month_short[i], german_month_short[i], buf, buflen );
	  }
  }
  DisplayCommon(buf, command, retbuf, bufsize);
  free( buf );
}

void DisplayRedraw( void )
{				
	for( int row = 0; row < nDisplayRows; row++ )
	{
	  lcdSetCursorPos(0, row);
	  lcdSendString(pDisplayBuf+row*nDisplayCols, nDisplayCols );
	}
}


/*  This is the function which identifies this
  plugin as an software plugin. It gets called
  as soon as the plugin is loaded. BUT when init_dllex
  is defined this function gets called with NULL as the argument
  As init_dllex adds a better method of using the plugin. 

  In:
  TFunctions : The functions that girder exports for usage in the plugin
  
  Out:
  None
   
  Return:
  Int : the Plugin device number
  

*/
/*
extern "C" int WINAPI __export init_dll( TFunctions *gf )
{
  if ( gf!=NULL) {
    MessageBox(0, "This Plugin requires Girder 3.0.22+", "Error", MB_ICONERROR);
    return GIR_ERROR; 
  }
  else {
    return PLUGINNUM;
  }
}
*/

/* If this function is defined init_dll will never be called BUT
   init_dll has to be in here for the plugin to work!!!!! 

  In:
  Out:
  Return: Integer, the version of structure that this plugin uses.

*/
/*
extern "C" int WINAPI __export init_dllex()
{

  return 4;  // We want version 4 structure

}
*/
/* This function gets called right after the init_dllex has
   requested a specific type of header, this is then passed here.

   In: void *s : This hold a pointer to the requested struct
   Out:
   Return : integer : the plugin number
*/
/*
static char ArgPrefix[] = { (char)255, 0 };

extern "C" int WINAPI __export setsupportex(void *s)
{
  TFunctionsEx4 *p;

  p = (TFunctionsEx4 *) s;

  if ( p->dwSize != sizeof ( TFunctionsEx4 ) )
  {
  
    return GIR_ERROR; // ERROR 

  }
  
  SetCommand    = p->SetCommand;
  ReallocPchar  = p->ReallocPchar;
  ShowOSD      = p->ShowOSD;
  TargetEnum    = p->TargetEnum;
  I18NTranslate  = p->I18NTranslate;
  HideOSD      = p->HideOSD;
  StartOSDDraw  = p->StartOSDDraw;
  StopOSDDraw    = p->StopOSDDraw;
  TreePickerShow  = p->TreePickerShow;
  TargetHWND    = p->TargetHWND;
  SetGirderReg  = p->SetGirderReg;
  GetGirderReg  = p->GetGirderReg;
  ParseRegString  = p->ParseRegString;
  GetLinkName     = p->GetLinkName;
  RegisterCB    = p->RegisterCB;
  SetGirderStrReg = p->SetGirderStrReg;
  GetGirderStrReg = p->GetGirderStrReg;

  return PLUGINNUM;
}
*/
/* Called right before Girder unloads the plugin, make 
   sure you cleaned everything up !!

*/
/*
extern "C" void WINAPI __export close_dll()
{
  DisplayClose();

  Close_Config();
}
*/
/* This function will be called when an action should be executed!
  
  In: TCommand *command: The details of the command to be executed
  Out: PCHAR ReturnBuf: an reallocated buffer with 1000 bytes of space

  */
/*
extern "C" void WINAPI __export command_execute(TCommand *command, PCHAR ReturnBuf)
{
  (*DisplayActions[command->ivalue1].function)(command, ReturnBuf);
}
*/
/* When the user selects a different command in the treeview the
   plugin will be notified of this event via this function

   In : TCommand *command: the details of the new plugin
*/
/*
extern "C" void WINAPI __export command_changed(TCommand  *command)
{
  CurCommand = command;
  Update_Config();
}
*/

/* You should open a configuration dialog for the plugin
   when this function gets called.
  
  In : TCommand *command: the details of the new plugin
*/
/*
extern "C" void WINAPI __export command_gui(TCommand *command)
{

  CurCommand = command;

  Show_Config();

}
*/

/* You should disable / enable all control on the config
   dialog based on te bValue */
/*
extern "C" void WINAPI __export command_controls(_BOOL bValue)
{
  Enable_Config(bValue);
}
*/

/* You should erase all values from the configuration
   dialog. */
/*
extern "C" void WINAPI __export command_clear()
{
  Empty_Config();

}
*/

/* Return the pluginname 

   In: integer length : the max len of the name
   Out PCHAR buffer : the buffer you should fill

*/
/*
extern "C" void WINAPI __export name_dll(PCHAR buffer, int length)
{

  strncpy(buffer, PLUGINNAME, length);

}
*/

/* Called by windows */
BOOL WINAPI DllMain( HANDLE hModule, DWORD fdwreason,  LPVOID lpReserved )
{
  UNREFERENCED_PARAMETER(lpReserved);
  
  switch(fdwreason) {
  case DLL_PROCESS_ATTACH:
    hInstance=HINSTANCE(hModule);
    break;
  case DLL_THREAD_ATTACH:
    break;
  case DLL_THREAD_DETACH:
    break;
  case DLL_PROCESS_DETACH:
    break;
  }
  return TRUE;
}

// Girder Plugin Interface

// these don't require any involvement by the application
void gir_version(PCHAR data, int size) { strncpy(data,PLUGINNAME,size); }
void gir_name(PCHAR data, int size) { strncpy(data,PLUGINNAME,size); }
void gir_description(PCHAR data, int size) { strncpy(data,PLUGINNAME,size); }
int gir_devicenum() { return PLUGINNUM; }
int gir_requested_api(int max_api) { return GIR_API_VERSION; }
/*
// called to compare strings
int gir_compare(PCHAR orig, PCHAR recv) { return (strcmp(orig,recv)==0)?GIR_TRUE:GIR_FALSE; }
*/

// called to open the plugin
int gir_open(int gir_major_ver, int gir_minor_ver, int gir_micro_ver, p_functions api_functions )
{
  Functions = *api_functions;
  return GIR_TRUE;
}
// called to close the plugin
int gir_close()
{
  DisplayClose();
  Close_Config();
  return GIR_TRUE;
}

// called when an event triggers a command
int gir_event(p_command command, PCHAR eventstring, void *payload, int len, PCHAR status, int statuslen)
{
  (*DisplayActions[command->ivalue1].function)(command, status, statuslen);
  return retContinue;
}

// called to put up the command dialog
void gir_command_gui()
{
  Show_Config();
}

// called when a command is highlited in the tree
void gir_command_changed(p_command command)
{
  bool bValue = (command != 0);
  CurCommand = command;
  Update_Config();
  Enable_Config(bValue);
}

// called with "extra" information
int gir_info(int message, int wparam, int lparam)
{
    if( message == GIRINFO_POWERBROADCAST )
	{
		if( wparam == PBT_APMRESUMESUSPEND )
		{
			CHAR ret[100];
			if( pDisplayBuf )
			{
				LPSTR p = (LPSTR) malloc( nDisplayCols * nDisplayRows );
				memcpy( p, pDisplayBuf, nDisplayCols * nDisplayRows );
				DisplayClose();
				DisplayOpen( ret, 99, p);
				DisplayRedraw();
				free( p );
			}
		}
	}
	
	return GIR_TRUE;
}
